function [g] = ngrad(f,p,meth)
% p norm of gradient

switch meth
  case 'central4'
    [nx,ny] = size(f);
    rx = 3:nx-2;
    ry = 3:ny-2;
    v = 0*f;
    v(rx,ry) = -f(rx+2,ry)/12+f(rx+1,ry)*2/3-f(rx-1,ry)*2/3+f(rx-2,ry)/12;
    g1 = fill4(v);
    v(rx,ry) = -f(rx,ry+2)/12+f(rx,ry+1)*2/3-f(rx,ry-1)*2/3+f(rx,ry-2)/12;
    g2 = fill4(v);
    v(rx,ry) = -f(rx+2,ry+2)/12+f(rx+1,ry+1)*2/3-f(rx-1,ry-1)*2/3+f(rx-2,ry-2)/12;
    g3 = fill4(v)/sqrt(2);
    v(rx,ry) = -f(rx+2,ry-2)/12+f(rx+1,ry-1)*2/3-f(rx-1,ry+1)*2/3+f(rx-2,ry+2)/12;
    g4 = fill4(v)/sqrt(2);
    
    v(rx,ry) = (-f(rx+2,ry+1)+f(rx-2,ry-1))/12+(f(rx+1,ry+0/2)-f(rx-1,ry-0/2)+f(rx+1,ry+2/2)-f(rx-1,ry-2/2))*1/3;
    g5 = fill4(v)*2/sqrt(5);
    v(rx,ry) = (-f(rx+1,ry+2)+f(rx-1,ry-2))/12+(f(rx+0/2,ry+1)-f(rx-0/2,ry-1) + f(rx+2/2,ry+1)-f(rx-2/2,ry-1))*1/3;
    g6 = fill4(v)*2/sqrt(5);
    v(rx,ry) = (-f(rx-1,ry+2)+f(rx+1,ry-2))/12+(f(rx-0/2,ry+1)-f(rx+0/2,ry-1) + f(rx-2/2,ry+1)-f(rx+2/2,ry-1))*1/3;
    g7 = fill4(v)*2/sqrt(5);
    v(rx,ry) = (-f(rx-2,ry+1)+f(rx+2,ry-1))/12+(f(rx-1,ry+0/2)-f(rx+1,ry-0/2) + f(rx-1,ry+2/2)-f(rx+1,ry-2/2))*1/3;
    g8 = fill4(v)*2/sqrt(5);
    g = pnrm({g1,g2,g3,g4,g5,g6,g7,g8},p);
  case 'central'
    gx = 0*f;
    gy = 0*f;
    g1 = 0*f;
    g2 = 0*f;
    gx(2:end-1,:) = (f(3:end,:) - f(1:end-2,:))/2;  
    gy(:,2:end-1) = (f(:,3:end) - f(:,1:end-2))/2;  
    g1(2:end-1,2:end-1) = (f(3:end,3:end) - f(1:end-2,1:end-2))*sqrt(2)/4;  
    g2(2:end-1,2:end-1) = (f(3:end,1:end-2) - f(1:end-2,3:end))*sqrt(2)/4;
    gx = fill(gx);
    gy = fill(gy);
    g1 = fill(g1);
    g2 = fill(g2);
    g = pnrm({gx,gy,g1,g2},p);
  case 'forward'
    gx = 0*f;
    gy = 0*f;
    g_x = 0*f;
    g_y = 0*f;
    gur = 0*f;
    gul = 0*f;
    gdr = 0*f;
    gdl = 0*f;
    gx(2:end-1,:) = f(3:end,:) - f(2:end-1,:);  
    g_x(2:end-1,:) = f(1:end-2,:) - f(2:end-1,:);  
    gy(:,2:end-1) = f(:,3:end) - f(:,2:end-1);  
    g_y(:,2:end-1) = f(:,1:end-2) - f(:,2:end-1);
    gur(2:end-1,2:end-1) = f(2:end-1,2:end-1) - f(3:end,3:end);  
    gul(2:end-1,2:end-1) = f(2:end-1,2:end-1) - f(3:end,1:end-2);
    gdr(2:end-1,2:end-1) = f(2:end-1,2:end-1) - f(1:end-2,3:end);  
    gdl(2:end-1,2:end-1) = f(2:end-1,2:end-1) - f(1:end-2,1:end-2);  
    gx = fill(gx);
    gy = fill(gy);
    g_x = fill(g_x);
    g_y = fill(g_y);
    gur = fill(gur)/sqrt(2);
    gul = fill(gul)/sqrt(2);
    gdr = fill(gdr)/sqrt(2);
    gdl = fill(gdl)/sqrt(2);
    g = pnrm({gx,gy,g_x,g_y,gur,gul,gdr,gdl},p);
  case 'diag'
    gr = 0*f;
    gl = 0*f;
    gr(1:end-1,1:end-1) = f(2:end,2:end) - f(1:end-1,1:end-1);  
    gl(1:end-1,1:end-1) = f(2:end,1:end-1) - f(1:end-1,2:end);  
    gr = fill(gr)/sqrt(2);
    gl = fill(gl)/sqrt(2);
    g = pnrm({gl,gr},p);
  case 'xy'
    gx = 0*f;
    gy = 0*f;
    gx(1:end-1,:) = f(2:end,:) - f(1:end-1,:);  
    gx(end,:) = gx(end-1,:);
    gy(:,1:end-1) = f(:,2:end) - f(:,1:end-1);  
    gy(:,end) = gx(:,end-1);
    gx = fill(gx);
    gy = fill(gy);
    g = pnrm({gx,gy},p);
  otherwise
    g = 0*f;
end

function x = pnrm(c,p)
% p norm

nc = length(c);
x = 0*c{1};

for k=1:nc
   x = x + abs(c{k}).^p;
end
% need 2 as 2 components contribute
x = (2*x/nc).^(1./p);

function y = fill(g)
y = g;
y(1,2:end-1) = y(2,2:end-1);
y(end,2:end-1) = y(end-1,2:end-1);
y(2:end-1,1) = y(2:end-1,2); 
y(2:end-1,end) = y(2:end-1,end-1);
y(1,1) = y(2,2);
y(end,1) = y(end-1,2);
y(1,end) = y(2,end-1);
y(end,end) = y(end-1,end-1);

function y = fill4(g)
[nx,ny] = size(g);
rx = 3:nx-2;
ry = 3:ny-2;
y = g;
y(rx,2) = y(rx,3);
y(rx,1) = y(rx,3);
y(rx,end-1) = y(rx,end-2);
y(rx,end) = y(rx,end-2);
y(2,ry) = y(3,ry);
y(1,ry) = y(3,ry);
y(end-1,ry) = y(end-2,ry);
y(end,ry) = y(end-2,ry);
y(1:2,1:2) = y(3,3);
y(1:2,end-1:end) = y(3,end-2);
y(end-1:end,1:2) = y(end-2,3);
y(end-1:end,end-1:end) = y(end-2,end-2);