% get location/colormap from Kalles Fractaler file
fn = 'a1.kfr';
fnpng = 'conf4';
dirpng = './png';
% if set do not recompute iterations/distance
doNotRecompute = 0;
% oversampling
scale = 1;
% which figs to show, 0 not, 1 yes, 2 yes + write png
doDE =      2;
doNDE =     0;
doDDE =     0;
doEntrIt =  0;
doEntrDE =  0;
doEntrNDE = 0;
doIters =   0;
% fish eye zoom-out. If 1 no fish eye, feye id >1, anti-feye if 0< <1,
% weird if <0.
magn2 = 1;

% conformal map of c (as in z=z^2+c)
doConformalMap = 0;
% define any analytical function defined at x=0 and derivative for DE
%[fconformal fconformalP] = fzn(6);
fconformal = @(x) x+1./(1+x);
fconformalP = @(x) 1-1./(1+x).^2;
        
% for extended precision toolbox mp https://www.advanpix.com/
%mp.Digits(64);
%[x0 y0 magn nmax cmap] = readKFRMP(fn);
[x0 y0 magn nmax cmap] = readKFR(fn);

% resolution
nx = 640*scale*1;
ny = 360*scale*1;
% escape radius
R = 1e5;
% number of processors
nproc = 4;

try
    iptsetpref('ImshowBorder','tight');
catch ex
end
if(exist('ex'))
    imflush = @()sin(0);
else
    imflush = @()iptsetpref('ImshowBorder','tight');
end

if(~doNotRecompute)
    % make grid and put c on grid
    dx = 3.5/magn * magn2;
    dy = ny/nx * dx;
    xmin = x0 - dx;
    xmax = x0 + dx;
    ymin = y0 - dy;
    ymax = y0 + dy;
    xgv = linspace(xmin,xmax,nx);
    ygv = linspace(ymin,ymax,ny);
    [xg,yg] = ndgrid(xgv,ygv);
    c = xg + 1i*yg;
    fapply = [];
    fapplyP = [];
    % fish eye zoom:
    c0 = x0+1i*y0;
    Rc = dy;
    fzoom = @(c)(c0+(c-c0).*exp(log(magn2)/Rc^2*(abs(c-c0).^2-Rc^2)));
    fzoomP = @(c) (1+log(magn2)/Rc^2*abs(c-c0).^2).*exp(log(magn2)/Rc^2*(abs(c-c0).^2-Rc^2));
    if(magn2~=1)
        fapply = fzoom;
        fapplyP = fzoomP;
    end

    if(doConformalMap)
        Rcc0 = Rc/1.25;
        %c1 = -1.2558 +0.3810367*1i;
        %Rcc1 = 1/400;
        c1 = 0;
        Rcc1 = 1.5;
        fscale = @(x) (x-c0)*Rcc1/Rcc0 + c1;
        funscale = @(x) (x-c1)*Rcc0/Rcc1 + c0;;
        fc = @(x) fconformal(x) - fconformal(0);
        fapply = @(x) funscale(fc(fscale(x)));
        fapplyP = @(x) fconformalP(fscale(x));
    end

    % divide over processors
    nn = length(c(:));
    for k=1:nproc
        pind{k} = k:nproc:nn;
        pc{k} = c(pind{k}); 
    end
    % calculate
    tic
    parfor k=1:nproc
        [pm{k} pmd{k} pmdd{k}] = procRegion(pc{k},nmax,R,k,fapply,fapplyP);
    end
    toc

    % reassemble parts
    m = 0*c;
    md = 0*c;
    mdd = 0*c;
    for k=1:nproc
        m(pind{k}) = pm{k};
        md(pind{k}) = pmd{k};
        mdd(pind{k}) = pmdd{k};
    end

    m = reshape(m,size(c))';
    md = reshape(md,size(c))';
    mdd = reshape(mdd,size(c))';
end

% numerical distance estimator
% norm used usually 2, -2 looks interesting
pde = 2; 
gmethod = 'diag';
h = 2*dx/nx;
% find inside points
indIn = find(m(:)>nmax);
g = ngrad(m,pde,gmethod);
g = g/h;
ind = find(g(:)>0);
mnde = g*0;
% numerical distance estimate
mnde(ind) = 1./(log(2)*g(ind));
mnde(indIn) = 0;
% error excluding boundary effects
y1 = mnde(3:end-2,3:end-2);
y2 = md(3:end-2,3:end-2);
merr = mean((norm(y1(:)-y2(:)))/norm(y2(:)));
fprintf('de/nde error: %.4f percent\n',100*merr);

%%%%%%%%%%%%%%%%%%%%%%%%%

% analytic DE
if(doDE)
    figure(1);
    set(gcf(),'name','Analytic DE');
    ftd = @(x)(-log(1./(x+h^2)));
    cdiv = 1;
    rgb = makeRGB(ftd(md),cmap,indIn,cdiv);
    rgbsc = imresize(rgb,1/scale);
    imflush();
    imshow(rgbsc);
    if(doDE==2)
        fn = sprintf('%s/DE%s.png',dirpng,fnpng);
        imwrite(rgbsc,fn);
    end
end

% numeric DE
if(doNDE)
    figure(2);
    set(gcf(),'name','Numerical DE');
    ftd = @(x)(log(1./(x+1e12*h^2)));
    cdiv = 1;
    rgb = makeRGB(ftd(mnde),cmap,indIn,cdiv);
    rgbsc = imresize(rgb,1/scale);
    imflush();
    imshow(rgbsc);
    if(doNDE==2)
        fn = sprintf('%s/NDE%s.png',dirpng,fnpng);
        imwrite(rgbsc,fn);
    end
end

% analytic DDE
if(doDDE)
    figure(3);
    set(gcf(),'name','Analytical DDE');
    ftd = @(x)(-x.^.2);
    cdiv = 1;
    rgb = makeRGB(ftd(mdd),cmap,indIn,cdiv,[.5 .5 .5]);
    rgbsc = imresize(rgb,1/scale);
    imflush();
    imshow(rgbsc);
    if(doDDE==2)
        fn = sprintf('%s/DDE%s.png',dirpng,fnpng);
        imwrite(rgbsc,fn);
    end
end

% entropy of numerical grad
if(doEntrNDE)
    figure(4),
    set(gcf(),'name','Entr Numerical DE');
    fentr = 5;
    ft = @(x)(exp(-fentr*x));
    ennde = emap(mnde);
    ft = @(x)(exp(-fentr*x));
    rgb = makeRGB(ft(ennde),cmap,indIn);
    rgbsc = imresize(rgb,1/scale);
    imflush();
    imshow(rgbsc);
    if(doEntrNDE==2)
        fn = sprintf('%s/EntrNDE%s.png',dirpng,fnpng);
        imwrite(rgbsc,fn);
    end
end

% entropy of analytic DE
if(doEntrDE)
    figure(5);
    set(gcf(),'name','Entr Analytic DE');
    fentr = 5;
    ft = @(x)(exp(-fentr*x));
    enade = emap(md);
    rgb = makeRGB(ft(enade),cmap,indIn);
    rgbsc = imresize(rgb,1/scale);
    imflush();
    imshow(rgbsc);
    if(doEntrDE==2)
        fn = sprintf('%s/EntrDE%s.png',dirpng,fnpng);
        imwrite(rgbsc,fn);
    end
end

% entropy of iterations
if(doEntrIt)
    figure(6);
    set(gcf(),'name','Entr iterations');
    fentr = 5;
    ft = @(x)(exp(-fentr*x));
    enit = emap(m);
    rgb = makeRGB(ft(enit),cmap,indIn);
    rgbsc = imresize(rgb,1/scale);
    imflush();
    imshow(rgbsc);
    if(doEntrIt==2)
        fn = sprintf('%s/EntrIt%s.png',dirpng,fnpng);
        imwrite(rgbsc,fn);
    end
end

% iterations
if(doIters)
    figure(7);
    set(gcf(),'name','Tterations');
    % expand colormap
    ncols1 = size(cmap,1);
    ncols2 = ncols1*4;
    cmap2 = interp1(1:ncols1, cmap, linspace(1,ncols1,ncols2), 'linear');
    ft = @(x)(-x);
    cdiv = .1;
    rgb = makeRGB(ft(m),cmap2,indIn,cdiv);
    rgbsc = imresize(rgb,1/scale);
    imflush();
    imshow(rgbsc);
    if(doIters==2)
        fn = sprintf('%s/Iters%s.png',dirpng,fnpng);
        imwrite(rgbsc,fn);
    end
end