% read in Kalles Fractaler binary file
fn = 'a1.kfb';
fnpng = 'tmp.png';
scale = 2; % shrink factor

[width,height,counts,trans,maxiter,parts,keys] = readKFB(fn);
indIn = find(counts(:)==maxiter);
cmap = reshape(keys,3,parts)'/255.;
cmap = flipud(fliplr(cmap));

m = (counts - trans);

% numerical distance estimator
pde = 2; % norm used  
g = ngrad(m,pde,'diag');
ind = find(g(:)>0);
md = g;
md(ind) = 1./g(ind);
md(indIn) = 0;
% -2 norm
pde = -2; 
g = ngrad(m,pde,'diag');
ind = find(g(:)>0);
md_2 = g;
md_2(ind) = 1./g(ind);
md_2(indIn) = 0;
% entropy
em = emap(m);
emd = emap(md);

try
    iptsetpref('ImshowBorder','tight');
catch ex
end
if(exist('ex'))
    imflush = @()sin(0);
else
    imflush = @()iptsetpref('ImshowBorder','tight');
end

figure(1)
set(gcf(),'name','Entropy iters');
fentr = 7;
ft = @(x)(exp(-fentr*x));
rgb = makeRGB(ft(em),cmap,indIn);
rgbsc = imresize(rgb,1/scale);
imflush();
imshow(rgbsc);
%imwrite(rgbsc,fnpng);

figure(2);
set(gcf(),'name','Entropy DE');
fentr = 7;
ft = @(x)(exp(-fentr*x));
rgb = makeRGB(ft(emd),cmap,indIn);
rgbsc = imresize(rgb,1/scale);
imflush();
imshow(rgbsc);

figure(3);
set(gcf(),'name','Numerical DE');
ftd = @(x)(-log(1./(x+1e-10)));
rgb = makeRGB(ftd(md),cmap,indIn);
rgbsc = imresize(rgb,1/scale);
imflush();
imshow(rgbsc);

figure(4);
set(gcf(),'name','Numerical -2 norm DE');
ftd = @(x)(-log(1./(x+1e-10)));
rgb = makeRGB(ftd(md_2),cmap,indIn);
rgbsc = imresize(rgb,1/scale);
imflush();
imshow(rgbsc);


